use cgmath::{Vector2, Vector3, Zero};
use utilities::{impl_reprc, prelude::*};
use vulkan_rs::prelude::*;

impl_reprc!(
    #[derive(Copy, Debug)]
    pub struct PositionColorUV {
        #[assume_reprc]
        pub position: Vector3<f32>,

        #[assume_reprc]
        filler_p: f32,

        #[assume_reprc]
        pub uv: Vector2<f32>,

        #[assume_reprc]
        filler_u: [f32; 2],
    }
);

impl PositionColorUV {
    pub fn new(position: Vector3<f32>, uv: Vector2<f32>) -> Self {
        Self {
            position,
            filler_p: 1.0,

            uv,
            filler_u: [0.0, 0.0],
        }
    }
}

impl Default for PositionColorUV {
    fn default() -> Self {
        PositionColorUV {
            position: Vector3::zero(),
            filler_p: 1.0,

            uv: Vector2::zero(),
            filler_u: [0.0, 0.0],
        }
    }
}

impl VertexInputDescription for PositionColorUV {
    fn attributes() -> Vec<VkVertexInputAttributeDescription> {
        vec![
            // position
            VkVertexInputAttributeDescription {
                location: 0,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 0,
            },
            // uvs
            VkVertexInputAttributeDescription {
                location: 3,
                binding: 0,
                format: VK_FORMAT_R32G32_SFLOAT,
                offset: 16,
            },
        ]
    }
}
