use cgmath::{Vector3, Zero};
use utilities::{impl_reprc, prelude::*};
use vulkan_rs::prelude::*;

impl_reprc!(
    #[derive(Copy, Debug)]
    pub struct PositionOnly {
        #[assume_reprc]
        pub position: Vector3<f32>,

        #[assume_reprc]
        filler: f32,
    }
);

impl PositionOnly {
    pub fn new(position: Vector3<f32>) -> Self {
        Self {
            position,
            filler: 1.0,
        }
    }
}

impl Default for PositionOnly {
    fn default() -> Self {
        Self {
            position: Vector3::zero(),
            filler: 1.0,
        }
    }
}

impl VertexInputDescription for PositionOnly {
    fn attributes() -> Vec<VkVertexInputAttributeDescription> {
        vec![VkVertexInputAttributeDescription {
            location: 0,
            binding: 0,
            format: VK_FORMAT_R32G32B32_SFLOAT,
            offset: 0,
        }]
    }
}
